'use strict'

/**
 * None
 */
class None {

  /**
   * sign
   */
  sign () {}

  /**
   * verify
   */
  verify () {
    // this will never get called. but you looked.
  }

}

/**
 * Export
 */
module.exports = None
